import groovy.json.JsonSlurper
import groovy.json.JsonOutput
import groovy.transform.Field
import java.util.logging.FileHandler
import java.util.logging.Level
import java.util.logging.Logger
import java.util.logging.SimpleFormatter

/********************************/
/*         Configuration        */
/********************************/

CROWD_SERVER_URL = "http://localhost:8095/crowd"
CROWD_ADMIN_USERNAME = "admin"
CROWD_ADMIN_PASSWORD = "********"
APP_CREDENTIALS_MAP = [ "jira": "********", "confluence": "********" ]

LOG_FILE_PATH = "guest-users-aliases.log"

/*******************************/
/*           Main              */
/*******************************/

@Field Logger logger = Logger.getLogger("")

FileHandler fh = new FileHandler(LOG_FILE_PATH, true)
SimpleFormatter formatter = new SimpleFormatter()
fh.setFormatter(formatter)
logger.addHandler(fh)

logger.setLevel(Level.INFO)

logger.log(Level.INFO, "Starting script")

def globalGuestUsersAliasesMap = [:]

APP_CREDENTIALS_MAP.each{ appName, appPassword ->
    logger.log(Level.INFO, "Looking for ${appName} appId")  
    appId = fetchAppId(appName)
    if (appId){
        logger.log(Level.INFO, "Success. ${appName} appId is: ${appId}")
	logger.log(Level.INFO, "Looking for ${appName} guest users")
	buildGuestUsersAliasesMapByApp(appName, appPassword).each{ guestUserName, guestUserAlias ->
            if (!globalGuestUsersAliasesMap.containsKey(guestUserName)){
                globalGuestUsersAliasesMap[guestUserName]=["${appId}":guestUserAlias]
            } else{
                def aliasesMap = globalGuestUsersAliasesMap[guestUserName]
                aliasesMap["${appId}"]=guestUserAlias
                globalGuestUsersAliasesMap[guestUserName]=aliasesMap
            }
        }
   } else{
	logger.log(Level.INFO, "Failure. ${appName} appId is null")
   } 
}

logger.log(Level.INFO, "${globalGuestUsersAliasesMap.size()} guest users will have their aliases updated or created")

createAliases(globalGuestUsersAliasesMap)

logger.log(Level.INFO, "End of script")

/*******************************/
/*          Functions          */
/*******************************/

def createAliases(guestUsersAliasesMap){
    guestUsersAliasesMap.each{ key, value ->
	def url = "${CROWD_SERVER_URL}/rest/appmanagement/1/aliases/?user=${URLEncoder.encode(key)}"
	def body = JsonOutput.toJson(value).getBytes('UTF-8')
	logger.log(Level.INFO, "Sending HTTP PUT request to ${url}")
	logger.log(Level.INFO, "Body is ${new String(body)}")
	def connection = new URL(url).openConnection()
        connection.setRequestProperty('Authorization', "Basic ${(CROWD_ADMIN_USERNAME + ':' + CROWD_ADMIN_PASSWORD).bytes.encodeBase64().toString()}")
        connection.setRequestProperty('Content-Type', 'application/json')
        connection.setRequestProperty('Accept', 'application/json')
        connection.setRequestMethod('PUT')
        connection.setDoOutput(true)
        connection.getOutputStream().write(body)
	connection.getInputStream()
	logger.log(Level.INFO, "Response code is: ${connection.responseCode}")
    }
    
}

def buildGuestUsersAliasesMapByApp(appName, appPassword){
    def guestUsersAliasesMap = [:]
    def appId = fetchAppId(appName)
    if (appId){
        def guestUsers = fetchGuestUsers(appName, appPassword)
        guestUsers.each{ guestUser ->
            def realGuestUserName = fetchRealGuestUsername(appId, guestUser.name)
            if (realGuestUserName && guestUser.email){
                 if (!guestUsersAliasesMap.containsKey(realGuestUserName)){
                     guestUsersAliasesMap[realGuestUserName] = guestUser.email
                 } 
            } 
        }
    }
    return guestUsersAliasesMap
}

def fetchAppId(appName){
    def url = "${CROWD_SERVER_URL}/rest/appmanagement/1/application?name=${URLEncoder.encode(appName)}"
    logger.log(Level.INFO, "Sending HTTP GET request to ${url}")
    def connection = new URL(url).openConnection()
    connection.setRequestProperty('Authorization', "Basic ${(CROWD_ADMIN_USERNAME + ':' + CROWD_ADMIN_PASSWORD).bytes.encodeBase64().toString()}")
    connection.setRequestProperty('Accept', 'application/json')
    logger.log(Level.INFO, "Response code is: ${connection.responseCode}")
    if (connection.responseCode.equals(200)){
        def json = connection.inputStream.withCloseable { inStream ->
            new JsonSlurper().parse( inStream as InputStream ) 
        }
        return json.id
        
    } else{
        return null
    }
}

def fetchGuestUsers(appName, appPassword){
    def url = "${CROWD_SERVER_URL}/rest/usermanagement/1/search?entity-type=user&restriction=userType%3DGuest"
    def guestUsers = []
    logger.log(Level.INFO, "Sending HTTP GET request to ${url}")
    def connection = new URL(url).openConnection()
    connection.setRequestProperty('Authorization', "Basic ${(appName + ':' + appPassword).bytes.encodeBase64().toString()}")
    connection.setRequestProperty('Accept', 'application/json')
    logger.log(Level.INFO, "Response code is: ${connection.responseCode}")
    if (connection.responseCode.equals(200)){
        def json = connection.inputStream.withCloseable { inStream ->
            new JsonSlurper().parse( inStream as InputStream ) 
        }
        def guestUsernames = json.users.name
        guestUsernames.each{ userName ->
           def user = fetchUserWithMailAttribute(appName, appPassword, userName)
           if (user){
               guestUsers.add(user)
           }
        }
        
    }
    return guestUsers
}

def fetchUserWithMailAttribute(appName, appPassword, userName){
    def url = "${CROWD_SERVER_URL}/rest/usermanagement/1/user?username=${URLEncoder.encode(userName)}"
    logger.log(Level.INFO, "Sending HTTP GET request to ${url}") 
    def connection = new URL(url).openConnection()
    connection.setRequestProperty('Authorization', "Basic ${(appName + ':' + appPassword).bytes.encodeBase64().toString()}")
    connection.setRequestProperty('Accept', 'application/json')
    logger.log(Level.INFO, "Response code is: ${connection.responseCode}")
    if (connection.responseCode.equals(200)){
        def json = connection.inputStream.withCloseable { inStream ->
            new JsonSlurper().parse( inStream as InputStream ) 
        }
        return new Expando(name: json.name, email: json.email)
    } else {
        return null
    }
}

def fetchRealGuestUsername(appId, alias){
    if (!appId) return null
    def url = "${CROWD_SERVER_URL}/rest/appmanagement/1/aliases/${appId}/username?alias=${URLEncoder.encode(alias)}"
    logger.log(Level.INFO, "Sending HTTP GET request to ${url}")
    def connection = new URL(url).openConnection()
    connection.setRequestProperty('Authorization', "Basic ${(CROWD_ADMIN_USERNAME + ':' + CROWD_ADMIN_PASSWORD).bytes.encodeBase64().toString()}")
    connection.setRequestProperty('Accept', 'text/plain')
    logger.log(Level.INFO, "Response code is: ${connection.responseCode}")
    if (connection.responseCode.equals(200)){
        return connection.inputStream.text
    } else {
        return null
    }
}
